-module(exp).

-behaviour(gen_server).

%% API
-export([start_link/0, start_link_and_register/0]).

%% gen_server callbacks
-export([init/1, handle_call/3, handle_cast/2, handle_info/2,
         terminate/2, code_change/3]).

-define(SERVER, ?MODULE). 

-record(state, {}).

start_link() ->
    gen_server:start_link(?MODULE, [], []).

start_link_and_register() ->
    gen_server:start_link({local, ?MODULE}, ?MODULE, [], []).

init([]) ->
    %% process_flag(trap_exit, true),
    {ok, #state{}}.

handle_call(_Request, _From, State) ->
    Reply = ok,
    {reply, Reply, State}.

handle_cast(_Msg, State) ->
    {noreply, State}.

handle_info({ping, Sender}, State) ->
    Sender ! pong,
    {noreply, State};
handle_info({unlink, Owner}, State) ->
    io:fwrite("~p, unlinking owner: ~p...~n", [self(), Owner]),
    unlink(Owner),
    {noreply, State};
handle_info({'EXIT', From, Reason}, State) ->
    io:fwrite("~p got 'EXIT' from parent ~p with reason ~p~n", [self(), From, Reason]),
    {noreply, State};
handle_info(stop, State) ->
    io:format("Stopping~n"),
    {stop, normal, State};
handle_info(_Info, State) ->
    io:format("Strange info: ~p~n", [_Info]),
    {noreply, State}.

%%--------------------------------------------------------------------
%% @private
%% @doc
%% This function is called by a gen_server when it is about to
%% terminate. It should be the opposite of Module:init/1 and do any
%% necessary cleaning up. When it returns, the gen_server terminates
%% with Reason. The return value is ignored.
%%
%% @spec terminate(Reason, State) -> void()
%% @end
%%--------------------------------------------------------------------
terminate(_Reason, _State) ->
    io:fwrite("~p terminated. Reason: ~p~n", [self(), _Reason]),
    ok.

code_change(_OldVsn, State, _Extra) ->
    {ok, State}.
