-module(exp_sup_sup).

-behaviour(supervisor).

%% API
-export([start_link/0]).

%% Supervisor callbacks
-export([init/1]).

-export([test/0]).

-include_lib("alog/include/alog.hrl").

-define(SERVER, ?MODULE).

test_simple() ->
    N = 1000,
    {ok, Pid} = exp_sup:start_child(),
    io:fwrite("process_info(~p) -> ~p~n", [Pid, process_info(Pid)]),
    do_send(Pid, N),
    supervisor:terminate_child(?MODULE, exp_sup),
    do_send(Pid, N),
    do_receive(2*N),
    io:fwrite("whereis(exp_sup) -> ~p~n", [whereis(exp_sup)]),
    io:fwrite("process_info(~p) -> ~p~n", [Pid, process_info(Pid)]).

test() ->
    Pid = whereis(exp),
    N = 1000,
    io:fwrite("whereis(~p) -> ~p~n", [exp, whereis(exp)]),
    do_send(Pid, N),
    supervisor:terminate_child(?MODULE, exp_sup2),
    do_send(Pid, N),
    do_receive(2*N),
    io:fwrite("whereis(exp_sup) -> ~p~n", [whereis(exp_sup2)]),
    io:fwrite("whereis(~p) -> ~p~n", [exp, whereis(exp)]).

do_send(_Pid, 0) ->
    ok;
do_send(Pid, N) ->
    Pid ! {ping, self()},
    do_send(Pid, N-1).

do_receive(0) ->
    io:fwrite("received all~n", []),
    ok;
do_receive(N) ->
    receive
        pong ->
            io:fwrite("got ~pth~n", [N]),
            do_receive(N-1)
    after 1000 ->
            io:fwrite("never got ~p; stopping.~n", [N])
    end.
    
start_link() ->
    supervisor:start_link({local, ?SERVER}, ?MODULE, []).

init([]) ->
    AChild = {exp_sup, {exp_sup, start_link, []},
              permanent, 1, supervisor, [exp_sup]},

    AChild2 = {exp_sup2, {exp_sup2, start_link, []},
               permanent, 1, supervisor, [exp_sup2]},

    {ok, {{one_for_one, 1, 1}, [AChild, AChild2]}}.

